'****************************************************************************
' SVGALIB.BAS
' Programmed by Aaron Severn (Laeden)
'               rlsevern@idirect.com
'               www.geocities.com/SiliconValley/Peaks/9572/
' Sept. 27/97
'
' This code is public domain and may be used freely.  Just give me credit if
' you use this code or any portion of this code.  Don't give me credit if
' this code crashes your computer.  Otherwise, enjoy.
'
' The functions contained here do the following:
'       DetectVESA    - Check to see if VESA capable graphics card exists.
'                       Returns 1 if successful, 0 if failed.
'       GetPixel      - Returns the colour of a pixel located at (x, y).
'       PutPixel      - Sets a pixel at (x, y) to colour clr.
'       SetActPage    - Sets the active video memory page (for page flipping).
'       SetSVGAMode   - Sets the screen to one of the following SVGA modes:
'                               &H100 - 640x400x256
'                               &H101 - 640x480x256
'                               &H103 - 800x600x256
'                               &H105 - 1024x768x256
'                               &H107 - 1280x1024x256
'                       Returns 1 if successful, 0 if failed.
'       SetText       - Sets the screen to text mode.  SCREEN 0 doesn't work
'                       after using SVGA, use this instead.
'       SVGACircle    - Draw a circle of center (x, y) with radius r.
'       SVGAGet       - Graphical GET routine for SVGA compatible with mode
'                       13 sprites.
'       SVGALine      - Draw a line from (x1, y1) to (x2, y2).
'       SVGAPut       - Graphical PUT routine for SVGA compatible with mode
'                       13 sprites.  Includes 3 styles, 0 - standard put,
'                       1 - put without destroying background (colour 0 is
'                       ignored), and 2 - put under background (only if
'                       background colour is 0).
'       ViewPage      - Sets the visible video memory page (for page
'                       flipping).
'****************************************************************************

DEFINT A-Z

DECLARE FUNCTION DetectVESA ()
DECLARE FUNCTION GetPixel (x, y)
DECLARE SUB PutPixel (x, y, clr)
DECLARE SUB SetActPage (page)
DECLARE FUNCTION SetSVGAMode (mode)
DECLARE SUB SetText ()
DECLARE SUB SVGACircle (x, y, r, clr)
DECLARE SUB SVGAGet (x1, y1, x2, y2, array())
DECLARE SUB SVGALine (x1, y1, x2, y2, clr)
DECLARE SUB SVGAPut (x, y, array(), style)
DECLARE SUB ViewPage (page)

COMMON SHARED xRes, yRes, yOffset&, curBank, winGran

TYPE RegType
    AX AS INTEGER
    BX AS INTEGER
    CX AS INTEGER
    DX AS INTEGER
    BP AS INTEGER
    SI AS INTEGER
    DI AS INTEGER
    FLAGS AS INTEGER
    DS AS INTEGER
    ES AS INTEGER
END TYPE
          
DIM SHARED Regs AS RegType

'-=-=-=-=-=-=-=-=-=-=-=-=-=-DEMO CODE STARTS HERE-=-=-=-=-=-=-=-=-=-=-=-=-=-

RANDOMIZE TIMER
DIM sprite(201)

IF DetectVESA THEN                              'Check for VESA
  IF SetSVGAMode(&H101) = 0 THEN                'Check for/set screen mode
    PRINT "Error:  Screen mode not available."  'Shut down if mode is not
    SYSTEM                                      'available
  END IF
ELSE
  PRINT "Error:  Cannot detect VESA."           'Shut down if VESA is not
  SYSTEM                                        'present
END IF

FOR y = 0 TO yRes - 1                           'Fill the screen with pixels
   FOR x = 0 TO xRes - 1
      PutPixel x, y, x MOD 256
   NEXT x
NEXT y

SVGAGet 30, 10, 49, 29, sprite()                'Grab a sprite

SetActPage 1                                    'Change the active page

SVGALine 0, 0, xRes - 1, yRes - 1, 1            'Draw a line
SVGACircle xRes \ 2, yRes \ 2, yRes \ 2 - 1, 4  'Draw a circle

FOR i = 0 TO 25                                 'Randomly place 25 sprites
   x = INT(RND * (xRes - 20))                   'without disturbing the line
   y = INT(RND * (yRes - 20))                   'and circle (style 2)
   SVGAPut x, y, sprite(), 2
NEXT i

DO: LOOP UNTIL INKEY$ = ""
curPage = 0
DO
  curPage = curPage XOR 1                       'Flip the visible page
  ViewPage curPage                              'every half second.

  endTime! = TIMER + .5
  DO: LOOP UNTIL TIMER > endTime!
LOOP UNTIL INKEY$ <> ""

SetText                                         'Back to text mode
DEF SEG                                         'Reset default segment
SYSTEM                                          'Shut down

' Detect to see if a VESA compatible graphics card is present.
FUNCTION DetectVESA

DIM VESAInfo(127)

Regs.AX = &H4F00
Regs.ES = VARSEG(VESAInfo(0))
Regs.DI = VARPTR(VESAInfo(0))
CALL INTERRUPTX(&H10, Regs, Regs)

IF Regs.AX = &H4F THEN DetectVESA = 1 ELSE DetectVESA = 0

END FUNCTION

' Return the colour of a pixel.
FUNCTION GetPixel (x, y)

offset& = (y + yOffset&) * xRes + x
bank = offset& \ &H10000
offset& = offset& - &H10000 * bank
IF bank <> curBank THEN
  curBank = bank
  Regs.AX = &H4F05
  Regs.BX = 0
  Regs.DX = winGran * curBank
  CALL INTERRUPT(&H10, Regs, Regs)
END IF
GetPixel = PEEK(offset&)

END FUNCTION

' Set a pixel.
SUB PutPixel (x, y, clr)

offset& = (y + yOffset&) * xRes + x
bank = offset& \ &H10000
offset& = offset& - &H10000 * bank
IF bank <> curBank THEN
  curBank = bank
  Regs.AX = &H4F05
  Regs.BX = 0
  Regs.DX = winGran * curBank
  CALL INTERRUPT(&H10, Regs, Regs)
END IF
POKE offset&, clr

END SUB

' Just changes the pointer to the video memory but it's easier to use this
' way.
SUB SetActPage (page)

yOffset& = yRes * page

END SUB

' Set a SVGA mode.
FUNCTION SetSVGAMode (mode)

SELECT CASE mode
       CASE &H100: xRes = 640: yRes = 400
       CASE &H101: xRes = 640: yRes = 480
       CASE &H103: xRes = 800: yRes = 600
       CASE &H105: xRes = 1024: yRes = 768
       CASE &H107: xRes = 1280: yRes = 1024
       CASE ELSE: SetSVGAMode = 0: EXIT FUNCTION
END SELECT

DIM modeInfo(127)
Regs.AX = &H4F01
Regs.CX = mode
Regs.ES = VARSEG(modeInfo(0))
Regs.DI = VARPTR(modeInfo(0))
CALL INTERRUPTX(&H10, Regs, Regs)

winGran = 64 \ modeInfo(2)

Regs.AX = &H4F02
Regs.BX = mode
CALL INTERRUPT(&H10, Regs, Regs)

IF Regs.AX <> &H4F THEN
  SetSVGAMode = 0
  EXIT FUNCTION
END IF

DEF SEG = &HA000
curBank = 0
yOffset& = 0

SetSVGAMode = 1

END FUNCTION

' Sets text mode, SCREEN 0 doesn't work for some reason but this does.
SUB SetText

Regs.AX = &H3
CALL INTERRUPT(&H10, Regs, Regs)

END SUB

' Draws a circle using Bresenham's circle algorythm.
SUB SVGACircle (CX, cy, r, clr)

d = 3 - 2 * r
x = 0
y = r

DO
  PutPixel CX + x, cy + y, clr
  PutPixel CX + x, cy - y, clr
  PutPixel CX - x, cy + y, clr
  PutPixel CX - x, cy - y, clr
  PutPixel CX + y, cy + x, clr
  PutPixel CX + y, cy - x, clr
  PutPixel CX - y, cy + x, clr
  PutPixel CX - y, cy - x, clr

  IF d < 0 THEN
    d = d + (4 * x) + 6
  ELSE
    d = d + 4 * (x - y) + 10
    y = y - 1
  END IF

  x = x + 1
LOOP WHILE x <= y

END SUB

' Gets a sprite that is 100% compatible with mode 13 sprites.
SUB SVGAGet (x1, y1, x2, y2, array())

IF x1 >= xRes THEN x1 = xRes - 1
IF x2 >= xRes THEN x2 = xRes - 1
IF y1 >= yRes THEN y1 = yRes - 1
IF y2 >= yRes THEN y2 = yRes - 1

IF x1 < 0 THEN x1 = 0
IF x2 < 0 THEN x2 = 0
IF y1 < 0 THEN y1 = 0
IF y2 < 0 THEN y2 = 0

IF x2 < x1 THEN SWAP x2, x1
IF y2 < y1 THEN SWAP y2, y1

xSize = x2 - x1 + 1
ySize = y2 - y1 + 1
elems = ((xSize + 1) \ 2) * ySize + 1

IF UBOUND(array) < elems THEN ERROR (5)

array(0) = xSize * 8: array(1) = ySize

FOR y = y1 TO y2
   FOR x = x1 TO x2
      clr = GetPixel(x, y)
      DEF SEG = VARSEG(array(0))
      POKE 4 + (y - y1) * xSize + (x - x1), clr
      DEF SEG = &HA000
   NEXT x
NEXT y

END SUB

' Draws a line.
SUB SVGALine (x1, y1, x2, y2, clr)

xLength = x2 - x1
yLength = y2 - y1

d1X = SGN(xLength)
d1Y = SGN(yLength)
d2X = SGN(xLength)
d2Y = 0

xLen = ABS(xLength)
yLen = ABS(yLength)

IF xLen <= yLen THEN
  d2X = 0
  d2Y = SGN(yLength)
  xLen = ABS(yLength)
  yLen = ABS(xLength)
END IF

s = xLen \ 2
x = x1: y = y1

FOR i = 0 TO xLen
   PutPixel x, y, clr
   s = s + yLen
   IF s >= xLen THEN
     s = s - xLen
     x = x + d1X
     y = y + d1Y
   ELSE
     x = x + d2X
     y = y + d2Y
   END IF
NEXT i

END SUB

' Puts a sprite to the screen that was created either with SVGAGet or QBasic
' GET in mode 13.
SUB SVGAPut (x, y, array(), style)

xSize = array(0) \ 8: ySize = array(1)

FOR sY = y TO y + ySize - 1
   FOR sX = x TO x + xSize - 1
      DEF SEG = VARSEG(array(0))
      clr = PEEK(4 + (sY - y) * xSize + (sX - x))
      DEF SEG = &HA000
      SELECT CASE style
             CASE 0
                 PutPixel sX, sY, clr
             CASE 1
                 IF clr THEN PutPixel sX, sY, clr
             CASE 2
                 backClr = GetPixel(sX, sY)
                 IF backClr = 0 THEN PutPixel sX, sY, clr
      END SELECT
   NEXT sX
NEXT sY

END SUB

' Set the visible video memory page.
SUB ViewPage (page)

firstLine = page * yRes

Regs.AX = &H4F07
Regs.BX = 0
Regs.CX = 0
Regs.DX = firstLine
CALL INTERRUPT(&H10, Regs, Regs)

END SUB

